from wagtail import blocks


class VideoBlock(blocks.StructBlock):
    # TODO: Possible video Types for the user, upload file, add URL
    description = blocks.TextBlock()
    url = blocks.URLBlock()

    class Meta:
        icon = 'media'


class WebBasedTrainingBlock(blocks.StructBlock):
    description = blocks.TextBlock()
    url = blocks.URLBlock()

    class Meta:
        icon = 'media'


class PodcastBlock(blocks.StructBlock):
    description = blocks.TextBlock()
    url = blocks.URLBlock()

    class Meta:
        icon = 'media'


class CompetenceBlock(blocks.StructBlock):
    description = blocks.TextBlock()

    class Meta:
        icon = 'media'


class ExerciseBlock(blocks.StructBlock):
    description = blocks.TextBlock()

    class Meta:
        icon = 'media'


class DocumentBlock(blocks.StructBlock):
    description = blocks.TextBlock()

    class Meta:
        icon = 'media'


class KnowledgeBlock(blocks.StructBlock):
    description = blocks.TextBlock()

    class Meta:
        icon = 'media'
